%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BuildoptimalTable() generates the optimal solving table optimalTable for
%   the 2x2x2 cube, i.e., it contains the distance value (modulo 3) for
%   every possible cube state. Furthermore, the table is compressed in a
%   way that every entry in the table encodes 5 cube states distances,
%   converted the 5 numbers of {0,1,2} into a single number in base 10.
%   This table is the heart of solving the 2x2x2 cube optimally. The size
%   of the table is 734832-by-1.
% 
% The optimalTable is a global variable, which can not be reached in the
%   workspace by default. If you want it in your workspace, type
%   'global optimalTable' (without the apostrophes).
% 
% Depending on the hardware, building the optimalTable can run for seconds,
%   or even minutes. By default, the function OptimalTablePosition() loads
%   the table from optimalTable.mat, if it is not loaded or generated yet.
% 
% This function uses: BuildPermMoveTable(), BuildOriMoveTable()
% This function used in: -
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = BuildOptimalTable()

global optimalTable;
global permMoveTable;
global oriMoveTable;

PERMNUM = factorial(7);
ORINUM = power(3,6);
CUBENUM = PERMNUM * ORINUM;
UNKNOWN = -1; % Represents an unknown distance in the table

tic;

% permMoveTable and oriMoveTable is required for determining the neighbours
%   of the permutations and orientations
disp('Building move tables...')
BuildPermMoveTable();
BuildOriMoveTable();
disp('Done.')

% optTableFull contains the optimal distances (modulo 3) one by one, which
%   will be compressed further into optimalTable.
disp('Building optimal table... (this might take a while)');
optTableFull = int8(UNKNOWN*ones(CUBENUM,1));
optTableFull(1) = 0; % The solved state (with rank 0) has a distance 0
disp('Number of states requires 0 moves: 1');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BUILDING FULL OPTIMAL TABLE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% For every d distance value
% (Don't forget to shift by 1 the ranks because of matrix indexing)
for d = 0:10
    counter = 0; % Count the states which have exactly d+1 distance
    % For every possible permutation ranks
    for pr = 0:PERMNUM-1
        % For every moves (neighbours)
        for move = 1:9
            % The current cube's rank
            cubeRank = pr*ORINUM;
            % The neighbour's permutation rank
            cubeNeighbourPermRank = permMoveTable(pr+1,move)*ORINUM;
            % For every possible orientation ranks
            for or = 0:ORINUM-1
                % If the current cube has a d distance in the table
                if optTableFull(cubeRank+1) == d
                    % The neighbour's cube rank
                    cubeNeighbour = cubeNeighbourPermRank + ...
                        oriMoveTable(or+1,move);
                    % if the neighbour's distance is unknown...
                    if optTableFull(cubeNeighbour+1) == UNKNOWN
                        % ...then the neighbour's distance is d+1
                        optTableFull(cubeNeighbour+1) = d+1;
                        counter = counter + 1;
                    end
                end
                % The current cube is the next one (increase the rank by 1)
                cubeRank = cubeRank + 1;
            end
        end
    end
    disp(['Number of states requires ' num2str(d+1) ' moves: ' ...
        num2str(counter)]);
end

% Storing the distances in modulo 3 is enough; need conversion for avoiding
%   overflow during the compressing
optTableFull = uint8(mod(optTableFull,3));
disp('Done.');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPRESSING FULL OPTIMAL TABLE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Five successive numbers in optTableFull is a single number in base 3.
%   They can be stored in a single byte as a number in base 10
disp('Compressing optimal table...');
optimalTable = uint8(zeros(CUBENUM/5,1));
c = 1;
for i = 1:5:CUBENUM
    % Converting from base 3 to base 10
    optimalTable(c) = 81*optTableFull(i)   + 27*optTableFull(i+1) + ...
                       9*optTableFull(i+2) +  3*optTableFull(i+3) + ...
                         optTableFull(i+4);
    c = c+1;
end
disp('Done.');

toc;

end
