%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BuildPermMoveTable() builds the permutation movement table permMoveTable
%   as a global variable. For a given permutation rank i (0 <= i < 7!), the
%   table gives the its j-neighbour (1 <= j <= 9) permutation's rank. j
%   represents a rotation, where j = 1,...,9 means a single turn of U, U2, 
%   Ui, R, R2, Ri, F, F2, Fi, respectively. The size of permMoveTable is
%   7!-by-9.
%  
% The rank of a permutation is its ordinal number in the list of every
%   possible permutations with the same size (lexicographically ordered).
% 
% This function uses: PRotate()
% This function used in: BuildOptimalTable(), Solve()
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = BuildPermMoveTable()

global permMoveTable;

ROWNUM = factorial(7);
permMoveTable = -1*ones(ROWNUM,9);

% for every entry in the table
for pn = 0:ROWNUM-1
        
    % Calculate the (pn-1)-th permutation
    % (-1 because of matrix indexing)
    permRank = pn;
    perm = zeros(1,7);
    perm(7) = 1;
    for i = 6:-1:1
      perm(i) = 1 + mod(permRank,8-i);
      permRank = floor(permRank / (8-i));
      for j = i+1:7
        if perm(j) >= perm(i)
            perm(j) = perm(j) + 1;
        end
      end
    end

    % For every neighbour
    for move = 1:9
        % move-neighbour permutation
        permNew = PRotate(perm,move);
        % Calculate its rank
        permNewRank = 0;
        for i = 1:6
          permNewRank = permNewRank * (7-i+1);
          for j = i+1:7
            if permNew(i) > permNew(j)
                permNewRank = permNewRank + 1;
            end
          end 
        end
        % Update the table
        % (Don't forget to shift by 1 because of matrix indexing)
        permMoveTable(pn+1,move) = permNewRank;

    end
end

end
