%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve() solves a scrambled cube, printing out the solution to the screen.
% 
% Input:
%   cubeState - a scrambled cube: a 2-by-7 matrix
%
% Examples:
%   Solve([5 4 3 2 1 7 6; 1 1 1 2 0 0 1]);
%   Solve(Scramble('R U2 F Ui Ri F2 U F Ri U2 F'));
%   Solve(Stickers('owgbwowrgrygyrbgyobrw'));
% 
% This function uses: BuildPermMoveTable(), BuildOriMoveTable(),
%   OptimalDistance()
% This function used in: -
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = Solve(cubeState)

% permMoveTable and oriMoveTable contains the neighbours of a given
%   permutation and orientation respectively
global permMoveTable;
global oriMoveTable;

ORINUM = power(3,6); % Number of orientations

% Check if we have those tables
if isempty(permMoveTable) || isempty(oriMoveTable)
    BuildPermMoveTable();
    BuildOriMoveTable();
end

% Rank the permutation of the given cube state
permRank = 0;
for i = 1:6
  permRank = permRank * (7-i+1);
  for j = i+1:7
    if cubeState(1,i) > cubeState(1,j)
        permRank = permRank + 1;
    end
  end 
end

% Rank the orientation of the given cube state
oriRank =    cubeState(2,1) +  3*cubeState(2,2) +  9*cubeState(2,3) + ...
          27*cubeState(2,4) + 81*cubeState(2,5)+ 243*cubeState(2,6);

% Calculate the rank of the cube state
cubeRank = permRank*ORINUM + oriRank;

% Read its distance from the solved state (modulo 3)
% (Don't forget to shift by 1 because of matrix indexing)
d = OptimalDistance(cubeRank + 1);

Solution = [];

% While we don't reach the solved state
% (Don't forget to shift by 1 the ranks because of matrix indexing)
while(cubeRank ~= 0)
    % Look at all the neighbours
    for mv = 1:9
        % Calculate the rank of the neighbour cube state
        permRankNeighbour = permMoveTable(permRank+1,mv);
        oriRankNeighbour = oriMoveTable(oriRank+1,mv);
        cubeRankNeighbour = permRankNeighbour*ORINUM + oriRankNeighbour;
        
        % If a neighbour is closer to the solved state
        if OptimalDistance(cubeRankNeighbour+1) == mod(d-1,3)
            % We found the next state closer to the solved state
            Solution = [Solution mv]; %#ok<*AGROW>
            % The neighbour will be the current cube state in the next
            %   iteration
            permRank = permRankNeighbour;
            oriRank = oriRankNeighbour;
            cubeRank = permRank*ORINUM + oriRank;
            % Decrease the distance (modulo 3)
            d = mod(d-1,3);
            break;
        end
        
    end
end

% Rewrite the solution into a readable string
SolutionStr = '';
for i = 1:size(Solution,2)
    if Solution(i) == 1
        SolutionStr = [SolutionStr, 'U '];
    elseif Solution(i) == 2
        SolutionStr = [SolutionStr, 'U2 '];
    elseif Solution(i) == 3
        SolutionStr = [SolutionStr, 'Ui '];
    elseif Solution(i) == 4
        SolutionStr = [SolutionStr, 'R '];
    elseif Solution(i) == 5
        SolutionStr = [SolutionStr, 'R2 '];
    elseif Solution(i) == 6
        SolutionStr = [SolutionStr, 'Ri '];
    elseif Solution(i) == 7
        SolutionStr = [SolutionStr, 'F '];
    elseif Solution(i) == 8
        SolutionStr = [SolutionStr, 'F2 '];
    elseif Solution(i) == 9
        SolutionStr = [SolutionStr, 'Fi '];
    else
        error('Writing Error');
    end
end

% Display the solution and its length
disp(['Solution: ', SolutionStr, '(' num2str(size(Solution,2)) 'f*)']);

end