%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stickers() converts a cube represented by its stickers into a cube state
%   format.
% 
% Input:
%   colorStr - a string with a length of 21 characters (a 1-by-21 char
%   matrix)
% 
% Output:
%   cubeState - a cube state: a 2-by-7 matrix
% 
% Used functions: -
% 
% How to use:
% The string must contain 7 triplets (continuously), each triplet is made
% of three letters. A letter represents a color, which can be {w,y,g,b,o,r}
% by default. Each color triplet represents a corner cubie on the cube.
% 
% The colors of the faces by default:
%   up side     = white,
%   down side   = yellow,
%   front side  = green,
%   back side   = blue,
%   left side   = orange,
%   right side  = red.
% This information is encoded in the row:
%       U = 'w'; D = 'y'; F = 'g'; B = 'b'; L = 'o'; R = 'r';
% If you want to change that to match with your cube, e.g., up side is
% purple, then rewrite U = 'w'; to U = 'p';
% 
% The order of the triplets to read: ULB,UBR,URF,UFL,DRB,DFR,DLF. By
% default, DBL (the fixed corner) is the yellow-blue-orange, with its
% yellow sticker facing down.
% 
% First read the color of the cubie at the ULB place, in U-L-B order. The
% next color triplet is the color of the cubie at the UBR place, in U-B-R
% order. And so on. Every cubie is read in clockwise, starting with its
% up or down sticker. By default, the solved cube's color string is
% 'wobwbrwrgwgoyrbygryog'.
% A solved cube followed by a single U move:
% 'wgowobwbrwrgyrbygryog'
% A scrambled cube with the scramble R U2 Fi R2 Ui:
% 'ogybrwobwgryrbywgorgw'
%
% Example:
%   cubeState = Stickers('byrwobgwryogbrwwgogry');
% 
% This function uses: -
% This function used in: -
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function cubeState = Stickers(colorStr)

%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% REWRITE THIS LINE if you have a cube with different colors
U = 'w'; D = 'y'; F = 'g'; B = 'b'; L = 'o'; R = 'r';
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

% Reshape the scrambled cube's string into a 3-by-7 char matrix
colorTable = reshape(colorStr,3,7);
% The solved cube's string in a 3-by-7 form for reference
solvedTable = [U L B; U B R; U R F; U F L; D R B; D F R; D L F]';

% The final cube state
cubeState = zeros(2,7);

% For every triplet in the scrambled cube
for i = 1:7
    % The i-th triplet in colorTable represents a cubie
    cubie = colorTable(:,i);
    % Search the cubie's original place and orientation
    for j = 1:7
        % If the cubie belongs to position j with correct orientation
        if isequal(solvedTable(:,j),cubie)
            cubeState(1,i) = j;
            cubeState(2,i) = 0;
        % If the cubie belongs to position j and oriented CW
        elseif isequal(solvedTable(:,j),circshift(cubie,2))
            cubeState(1,i) = j;
            cubeState(2,i) = 1;
        % If the cubie belongs to position j and oriented CCW
        elseif isequal(solvedTable(:,j),circshift(cubie,1))
            cubeState(1,i) = j;
            cubeState(2,i) = 2;
        end
    end
end

% Check if the stickers really encodes a solvable cube
if ~isequal(sort(cubeState(1,:)),1:7) || mod(sum(cubeState(2,:)),3) ~= 0
    disp(' ');
    disp('The stickers encode this cube state, which is wrong!');
    disp(cubeState);
    error('The cube is unsolvable!');
end

end